@@ -14,16 +14,14 @@ with NML; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA."""
 
 from nml import expression, generic, nmlop
-from nml.actions import action1, action2, action2var, action6, actionD, real_sprite
-from nml.ast import general
+from nml.actions import action0, action1, action2, action2real, action2var, action6, actionD, real_sprite
+from nml.ast import general, spriteblock
 
 
 class Action2Layout(action2.Action2):
-    def __init__(self, feature, name, pos, ground_sprite, sprite_list, param_registers):
+    def __init__(self, feature, name, pos, layout, param_registers):
         action2.Action2.__init__(self, feature, name, pos)
-        assert ground_sprite.type == Action2LayoutSpriteType.GROUND
-        self.ground_sprite = ground_sprite
-        self.sprite_list = sprite_list
+        self.layout = layout
         self.param_registers = param_registers
 
     def resolve_tmp_storage(self):
@@ -40,53 +38,10 @@ class Action2Layout(action2.Action2):
             reg.set_register(location)
 
     def write(self, file):
-        advanced = any(x.is_advanced_sprite() for x in self.sprite_list + [self.ground_sprite])
-        size = 5
-        if advanced:
-            size += self.ground_sprite.get_registers_size()
-        for sprite in self.sprite_list:
-            if sprite.type == Action2LayoutSpriteType.CHILD:
-                size += 7
-            else:
-                size += 10
-            if advanced:
-                size += sprite.get_registers_size()
-        if len(self.sprite_list) == 0:
-            size += 9
-
+        size = self.layout.get_size()
         regs = ["{} : register {:X}".format(reg.name, reg.register) for reg in self.param_registers]
         action2.Action2.write_sprite_start(self, file, size, regs)
-        if advanced:
-            file.print_byte(0x40 | len(self.sprite_list))
-        else:
-            file.print_byte(len(self.sprite_list))
-        self.ground_sprite.write_sprite_number(file)
-        if advanced:
-            self.ground_sprite.write_flags(file)
-            self.ground_sprite.write_registers(file)
-        file.newline()
-        if len(self.sprite_list) == 0:
-            file.print_dwordx(0)  # sprite number 0 == no sprite
-            for _ in range(0, 5):
-                file.print_byte(0)  # empty bounding box. Note that number of zeros is 5, not 6
-        else:
-            for sprite in self.sprite_list:
-                sprite.write_sprite_number(file)
-                if advanced:
-                    sprite.write_flags(file)
-                file.print_byte(sprite.get_param("xoffset").value)
-                file.print_byte(sprite.get_param("yoffset").value)
-                if sprite.type == Action2LayoutSpriteType.CHILD:
-                    file.print_bytex(0x80)
-                else:
-                    # normal building sprite
-                    file.print_byte(sprite.get_param("zoffset").value)
-                    file.print_byte(sprite.get_param("xextent").value)
-                    file.print_byte(sprite.get_param("yextent").value)
-                    file.print_byte(sprite.get_param("zextent").value)
-                if advanced:
-                    sprite.write_registers(file)
-                file.newline()
+        self.layout.write(file)
         file.end_sprite()
 
 
@@ -128,8 +83,12 @@ class Action2LayoutSprite:
             self.params[i]["register"] = None
         self.sprite_from_action1 = False
         self.palette_from_action1 = False
+        self.var10_for_sprite = None
+        self.var10_for_palette = None
 
     def is_advanced_sprite(self):
+        if self.feature == 0x04:
+            return True
         if self.palette_from_action1:
             return True
         return len(self.get_all_registers()) != 0
@@ -137,6 +96,12 @@ class Action2LayoutSprite:
     def get_registers_size(self):
         # Number of registers to write
         size = len(self.get_all_registers())
+        # Add station specific registers
+        if self.feature == 0x04:
+            if self.var10_for_sprite:
+                size += 1
+            if self.var10_for_palette:
+                size += 1
         # Add 2 for the flags
         size += 2
         return size
@@ -160,6 +125,11 @@ class Action2LayoutSprite:
         nextreg = "zoffset" if self.type == Action2LayoutSpriteType.BUILDING else "yoffset"
         if self.get_register(nextreg) is not None:
             flags |= 1 << 5
+        if self.feature == 0x04:
+            if self.var10_for_sprite is not None:
+                flags |= 1 << 6
+            if self.var10_for_palette is not None:
+                flags |= 1 << 7
         file.print_wordx(flags)
 
     def write_register(self, file, name):
@@ -179,6 +149,11 @@ class Action2LayoutSprite:
             self.write_register(file, "yoffset")
         if self.get_register("zoffset") is not None:
             self.write_register(file, "zoffset")
+        if self.feature == 0x04:
+            if self.var10_for_sprite is not None:
+                file.print_bytex(self.var10_for_sprite)
+            if self.var10_for_palette is not None:
+                file.print_bytex(self.var10_for_palette)
 
     def write_sprite_number(self, file):
         num = self.get_sprite_number()
@@ -257,6 +232,28 @@ class Action2LayoutSprite:
         self.params[name]["value"] = self.params[name]["validator"](name, value)
         self.params[name]["is_set"] = True
 
+    def _validate_offset(self, offset, spriteset, pos):
+        if len(offset) == 0:
+            offset = None
+        elif len(offset) == 1:
+            id_dicts = [
+                (spriteset.labels if spriteset else {}, lambda name, val, pos: expression.ConstantNumeric(val, pos))
+            ]
+            offset = action2var.reduce_varaction2_expr(
+                offset[0], action2var.get_scope(self.feature), self.extra_dicts + id_dicts
+            )
+            if spriteset and isinstance(offset, expression.ConstantNumeric):
+                generic.check_range(
+                    offset.value,
+                    0,
+                    len(real_sprite.parse_sprite_data(spriteset)) - 1,
+                    "offset within spriteset",
+                    pos,
+                )
+        else:
+            raise generic.ScriptError("Expected 0 or 1 parameter, got " + str(len(offset)), pos)
+        return offset
+
     def resolve_spritegroup_ref(self, sg_ref):
         """
         Resolve a reference to a (sprite/palette) sprite group
@@ -268,36 +265,28 @@ class Action2LayoutSprite:
         @rtype: L{Expression}
         """
         spriteset = action2.resolve_spritegroup(sg_ref.name)
-
-        if len(sg_ref.param_list) == 0:
-            offset = None
-        elif len(sg_ref.param_list) == 1:
-            id_dicts = [(spriteset.labels, lambda name, val, pos: expression.ConstantNumeric(val, pos))]
-            offset = action2var.reduce_varaction2_expr(
-                sg_ref.param_list[0], action2var.get_scope(self.feature), self.extra_dicts + id_dicts
-            )
-            if isinstance(offset, expression.ConstantNumeric):
-                generic.check_range(
-                    offset.value,
-                    0,
-                    len(real_sprite.parse_sprite_data(spriteset)) - 1,
-                    "offset within spriteset",
-                    sg_ref.pos,
-                )
-        else:
-            raise generic.ScriptError("Expected 0 or 1 parameter, got " + str(len(sg_ref.param_list)), sg_ref.pos)
-
+        offset = self._validate_offset(sg_ref.param_list, spriteset, sg_ref.pos)
         num = action1.get_action1_index(spriteset)
         generic.check_range(num, 0, (1 << 14) - 1, "sprite", sg_ref.pos)
         return expression.ConstantNumeric(num), offset
 
     def _validate_sprite(self, name, value):
         if isinstance(value, expression.SpriteGroupRef):
+            assert self.feature != 0x04
             self.sprite_from_action1 = True
             val, offset = self.resolve_spritegroup_ref(value)
             if offset is not None:
                 self.create_register(name, offset)
             return val
+        elif isinstance(value, StationSpriteset):
+            assert self.feature == 0x04
+            self.sprite_from_action1 = True
+            if value.offset is not None:
+                offset = self._validate_offset(value.offset, value.spriteset, value.pos)
+                if offset is not None:
+                    self.create_register(name, offset)
+            self.var10_for_sprite = value.var10
+            return expression.ConstantNumeric(0x42D)
         else:
             self.sprite_from_action1 = False
             if isinstance(value, expression.ConstantNumeric):
@@ -320,11 +309,21 @@ class Action2LayoutSprite:
 
     def _validate_palette(self, name, value):
         if isinstance(value, expression.SpriteGroupRef):
+            assert self.feature != 0x04
             self.palette_from_action1 = True
             val, offset = self.resolve_spritegroup_ref(value)
             if offset is not None:
                 self.create_register(name, offset)
             return val
+        elif isinstance(value, StationSpriteset):
+            assert self.feature == 0x04
+            self.palette_from_action1 = True
+            if value.offset is not None:
+                offset = self._validate_offset(value.offset, value.spriteset, value.pos)
+                if offset is not None:
+                    self.create_register(name, offset)
+            self.var10_for_palette = value.var10
+            return expression.ConstantNumeric(0x42D)
         else:
             self.palette_from_action1 = False
             if isinstance(value, expression.ConstantNumeric):
@@ -385,13 +384,144 @@ class Action2LayoutSprite:
         self.create_register(name, expression.Not(value).reduce())
 
 
-def get_layout_action2s(spritelayout, feature, spr_pos):
-    """
-    @param spr_pos: Position information of the sprite view.
-    @type  spr_pos: L{Position}
-    """
-    ground_sprite = None
-    building_sprites = []
+class ParsedSpriteLayout:
+    def __init__(self, registers=None):
+        self.ground_sprite = None
+        self.building_sprites = []
+        self.registers = registers or []
+        self.advanced = False
+
+    def get_size(self):
+        size = 5
+        if self.advanced:
+            size += self.ground_sprite.get_registers_size()
+        for sprite in self.building_sprites:
+            if sprite.type == Action2LayoutSpriteType.CHILD:
+                size += 7
+            else:
+                size += 10
+            if self.advanced:
+                size += sprite.get_registers_size()
+        if len(self.building_sprites) == 0 and not self.advanced:
+            size += 9
+        return size
+
+    def write(self, file):
+        if self.advanced:
+            file.print_byte(0x40 | len(self.building_sprites))
+        else:
+            file.print_byte(len(self.building_sprites))
+        self.ground_sprite.write_sprite_number(file)
+        if self.advanced:
+            self.ground_sprite.write_flags(file)
+            self.ground_sprite.write_registers(file)
+        file.newline()
+        if len(self.building_sprites) == 0 and not self.advanced:
+            file.print_dwordx(0)  # sprite number 0 == no sprite
+            for _ in range(0, 5):
+                file.print_byte(0)  # empty bounding box. Note that number of zeros is 5, not 6
+        else:
+            for sprite in self.building_sprites:
+                sprite.write_sprite_number(file)
+                if self.advanced:
+                    sprite.write_flags(file)
+                file.print_byte(sprite.get_param("xoffset").value)
+                file.print_byte(sprite.get_param("yoffset").value)
+                if sprite.type == Action2LayoutSpriteType.CHILD:
+                    file.print_bytex(0x80)
+                else:
+                    # normal building sprite
+                    file.print_byte(sprite.get_param("zoffset").value)
+                    file.print_byte(sprite.get_param("xextent").value)
+                    file.print_byte(sprite.get_param("yextent").value)
+                    file.print_byte(sprite.get_param("zextent").value)
+                if self.advanced:
+                    sprite.write_registers(file)
+                file.newline()
+
+    def process(self, spritelayout, feature, param_map, actions, var10map=None):
+        if not isinstance(param_map, list):
+            param_map = [param_map]
+
+        # Reduce all expressions, can't do that earlier as feature is not known
+        all_sprite_sets = []
+        layout_sprite_list = []  # Create a new structure
+        for layout_sprite in spritelayout.layout_sprite_list:
+            param_list = []
+            layout_sprite_list.append((layout_sprite.type, layout_sprite.pos, param_list))
+            for param in layout_sprite.param_list:
+                param_val = action2var.reduce_varaction2_expr(param.value, action2var.get_scope(feature), param_map)
+                if isinstance(param_val, expression.SpriteGroupRef):
+                    spriteset = action2.resolve_spritegroup(param_val.name)
+                    if not spriteset.is_spriteset():
+                        raise generic.ScriptError("Expected a reference to a spriteset.", param_val.pos)
+                    if feature == 0x04:
+                        assert var10map is not None
+                        param_val = var10map.translate(spriteset, param_val.param_list, param_val.pos)
+                    else:
+                        all_sprite_sets.append(spriteset)
+                param_list.append((param.name, param_val))
+
+        actions.extend(action1.add_to_action1(all_sprite_sets, feature, spritelayout.pos))
+
+        for type, pos, param_list in layout_sprite_list:
+            if type.value not in layout_sprite_types:
+                raise generic.ScriptError(
+                    "Invalid sprite type '{}' encountered. Expected 'ground', 'building', or 'childsprite'.".format(
+                        type.value
+                    ),
+                    type.pos,
+                )
+            sprite = Action2LayoutSprite(feature, layout_sprite_types[type.value], pos, param_map)
+            for name, value in param_list:
+                sprite.set_param(name, value)
+            self.registers.extend(sprite.get_all_registers())
+            if sprite.type == Action2LayoutSpriteType.GROUND:
+                if self.ground_sprite is not None:
+                    raise generic.ScriptError("Sprite layout can have no more than one ground sprite", spritelayout.pos)
+                self.ground_sprite = sprite
+            else:
+                self.building_sprites.append(sprite)
+
+        if self.ground_sprite is None:
+            if len(self.building_sprites) == 0:
+                # no sprites defined at all, that's not very much.
+                raise generic.ScriptError("Sprite layout requires at least one sprite", spritelayout.pos)
+            # set to 0 for no ground sprite
+            self.ground_sprite = Action2LayoutSprite(feature, Action2LayoutSpriteType.GROUND)
+            self.ground_sprite.set_param(expression.Identifier("sprite"), expression.ConstantNumeric(0))
+
+        self.advanced = any(x.is_advanced_sprite() for x in self.building_sprites + [self.ground_sprite])
+
+    def write_action_value(self, actions, act6, offset):
+        sprite_num = self.ground_sprite.get_sprite_number()
+        sprite_num, offset = actionD.write_action_value(sprite_num, actions, act6, offset, 4)
+        if self.advanced:
+            offset += self.ground_sprite.get_registers_size()
+
+        for sprite in self.building_sprites:
+            sprite_num = sprite.get_sprite_number()
+            sprite_num, offset = actionD.write_action_value(sprite_num, actions, act6, offset, 4)
+            if self.advanced:
+                offset += sprite.get_registers_size()
+            offset += 3 if sprite.type == Action2LayoutSpriteType.CHILD else 6
+
+        return offset
+
+    def parse_registers(self, varact2parser):
+        if self.registers:
+            for register_info in self.registers:
+                reg, expr = register_info[1], register_info[2]
+                if reg is None:
+                    continue
+                varact2parser.parse_expr(expr)
+                varact2parser.var_list.append(nmlop.STO_TMP)
+                varact2parser.var_list.append(reg)
+                varact2parser.var_list.append(nmlop.VAL2)
+                varact2parser.var_list_size += reg.get_size() + 2
+
+
+def get_layout_action2s(spritelayout, feature):
     actions = []
 
     if feature not in action2.features_sprite_layout:
@@ -409,67 +539,13 @@ def get_layout_action2s(spritelayout, feature, spr_pos):
     param_map = (param_map, lambda name, value, pos: action2var.VarAction2LoadCallParam(value, name))
     spritelayout.register_map[feature] = param_registers
 
-    # Reduce all expressions, can't do that earlier as feature is not known
-    all_sprite_sets = []
-    layout_sprite_list = []  # Create a new structure
-    for layout_sprite in spritelayout.layout_sprite_list:
-        param_list = []
-        layout_sprite_list.append((layout_sprite.type, layout_sprite.pos, param_list))
-        for param in layout_sprite.param_list:
-            param_val = action2var.reduce_varaction2_expr(param.value, action2var.get_scope(feature), [param_map])
-            param_list.append((param.name, param_val))
-            if isinstance(param_val, expression.SpriteGroupRef):
-                spriteset = action2.resolve_spritegroup(param_val.name)
-                if not spriteset.is_spriteset():
-                    raise generic.ScriptError("Expected a reference to a spriteset.", param_val.pos)
-                all_sprite_sets.append(spriteset)
-    actions.extend(action1.add_to_action1(all_sprite_sets, feature, spritelayout.pos))
-
-    temp_registers = []
-    for type, pos, param_list in layout_sprite_list:
-        if type.value not in layout_sprite_types:
-            raise generic.ScriptError(
-                "Invalid sprite type '{}' encountered. Expected 'ground', 'building', or 'childsprite'.".format(
-                    type.value
-                ),
-                type.pos,
-            )
-        sprite = Action2LayoutSprite(feature, layout_sprite_types[type.value], pos, [param_map])
-        for name, value in param_list:
-            sprite.set_param(name, value)
-        temp_registers.extend(sprite.get_all_registers())
-        if sprite.type == Action2LayoutSpriteType.GROUND:
-            if ground_sprite is not None:
-                raise generic.ScriptError("Sprite layout can have no more than one ground sprite", spritelayout.pos)
-            ground_sprite = sprite
-        else:
-            building_sprites.append(sprite)
-
-    if ground_sprite is None:
-        if len(building_sprites) == 0:
-            # no sprites defined at all, that's not very much.
-            raise generic.ScriptError("Sprite layout requires at least one sprite", spr_pos)
-        # set to 0 for no ground sprite
-        ground_sprite = Action2LayoutSprite(feature, Action2LayoutSpriteType.GROUND)
-        ground_sprite.set_param(expression.Identifier("sprite"), expression.ConstantNumeric(0))
+    layout = ParsedSpriteLayout()
+    layout.process(spritelayout, feature, param_map, actions)
 
     action6.free_parameters.save()
     act6 = action6.Action6()
 
-    advanced = any(x.is_advanced_sprite() for x in building_sprites + [ground_sprite])
-
-    offset = 4
-    sprite_num = ground_sprite.get_sprite_number()
-    sprite_num, offset = actionD.write_action_value(sprite_num, actions, act6, offset, 4)
-    if advanced:
-        offset += ground_sprite.get_registers_size()
-
-    for sprite in building_sprites:
-        sprite_num = sprite.get_sprite_number()
-        sprite_num, offset = actionD.write_action_value(sprite_num, actions, act6, offset, 4)
-        if advanced:
-            offset += sprite.get_registers_size()
-        offset += 3 if sprite.type == Action2LayoutSpriteType.CHILD else 6
+    layout.write_action_value(actions, act6, 4)
 
     if len(act6.modifications) > 0:
         actions.append(act6)
@@ -478,26 +554,16 @@ def get_layout_action2s(spritelayout, feature, spr_pos):
         feature,
         spritelayout.name.value + " - feature {:02X}".format(feature),
         spritelayout.pos,
-        ground_sprite,
-        building_sprites,
+        layout,
         param_registers,
     )
     actions.append(layout_action)
 
-    if temp_registers:
-        varact2parser = action2var.Varaction2Parser(feature, action2var.get_scope(feature))
-        for register_info in temp_registers:
-            reg, expr = register_info[1], register_info[2]
-            if reg is None:
-                continue
-            varact2parser.parse_expr(expr)
-            varact2parser.var_list.append(nmlop.STO_TMP)
-            varact2parser.var_list.append(reg)
-            varact2parser.var_list.append(nmlop.VAL2)
-            varact2parser.var_list_size += reg.get_size() + 2
+    varact2parser = action2var.Varaction2Parser(feature)
+    layout.parse_registers(varact2parser)
 
     # Only continue if we actually needed any new registers
-    if temp_registers and varact2parser.var_list:  # lgtm[py/uninitialized-local-variable]
+    if varact2parser.var_list:
         # Remove the last VAL2 operator
         varact2parser.var_list.pop()
         varact2parser.var_list_size -= 1
@@ -547,6 +613,135 @@ def make_empty_layout_action2(feature, pos):
     @return: The created sprite layout action2
     @rtype: L{Action2Layout}
     """
-    ground_sprite = Action2LayoutSprite(feature, Action2LayoutSpriteType.GROUND)
-    ground_sprite.set_param(expression.Identifier("sprite"), expression.ConstantNumeric(0))
-    return Action2Layout(feature, "@CB_FAILED_LAYOUT{:02X}".format(feature), pos, ground_sprite, [], [])
+    layout = ParsedSpriteLayout()
+    layout.ground_sprite = Action2LayoutSprite(feature, Action2LayoutSpriteType.GROUND)
+    layout.ground_sprite.set_param(expression.Identifier("sprite"), expression.ConstantNumeric(0))
+    return Action2Layout(feature, "@CB_FAILED_LAYOUT{:02X}".format(feature), pos, layout, [])
+
+
+class StationSpriteset(expression.Expression):
+    def __init__(self, spriteset, args, var10, pos=None):
+        expression.Expression.__init__(self, pos)
+        self.spriteset = spriteset
+        self.offset = args
+        self.var10 = var10
+
+
+class StationSpritesetVar10Map:
+    def __init__(self):
+        self.spritesets = {}
+        self.var10 = 1  # Reserving 0 for SPRITESET() (basic action2)
+
+    def translate(self, spriteset, args, pos):
+        if spriteset not in self.spritesets:
+            if self.var10 == 8:
+                raise generic.ScriptError("A station can't use more than 6 different sprite sets", pos)
+            self.spritesets[spriteset] = self.var10
+            self.var10 += 1 if self.var10 != 1 else 2  # Reserving 2 for custom foundations
+        return StationSpriteset(
+            None if isinstance(spriteset, int) else spriteset, args, self.spritesets[spriteset], pos
+        )
+
+    def append_mapping(self, mapping, feature, actions, default, custom_spritesets):
+        for spriteset, var10 in self.spritesets.items():
+            if not isinstance(spriteset, int):
+                if not spriteset.has_action2(feature):
+                    actions.extend(action1.add_to_action1([spriteset], feature, None))
+                    real_action2 = action2real.make_simple_real_action2(
+                        feature,
+                        spriteset.name.value + " - feature {:02X}".format(feature),
+                        None,
+                        action1.get_action1_index(spriteset),
+                    )
+                    actions.append(real_action2)
+                    spriteset.set_action2(real_action2, feature)
+                ref = expression.SpriteGroupRef(spriteset.name, [], None, spriteset.get_action2(feature))
+            else:
+                if spriteset > len(custom_spritesets):
+                    raise generic.ScriptError("Index out of range")
+                ref = custom_spritesets[spriteset]
+            # Skip default result
+            if ref == default:
+                continue
+            mapping[var10] = (ref, None)
+        return mapping
+
+
+def parse_station_layouts(feature, id, layouts):
+    var10map = StationSpritesetVar10Map()
+
+    # Add DEFAULT([offset]) to reference active spriteset, selected by basic action2
+    # and CUSTOM(index, [offset]) to reference a spriteset from the custom list
+    def parse_spriteset(name, args, pos, info):
+        if info:
+            if len(args) < 1:
+                raise generic.ScriptError("'{}' expects 1 or 2 parameters".format(name), pos)
+            if not isinstance(args[0], expression.ConstantNumeric):
+                raise generic.ScriptError("First parameter for '{}' must be a constant".format(name), pos)
+            return var10map.translate(args[0].value, args[1:], pos)
+        return StationSpriteset(None, args, info, pos)
+
+    default_param = [
+        (
+            {"DEFAULT": None, "CUSTOM": True},
+            lambda name, value, pos: expression.FunctionPtr(expression.Identifier(name, pos), parse_spriteset, value),
+        )
+    ]
+
+    actions = []
+    param_registers = []
+    parsed_layouts = []
+    varact2parser = action2var.Varaction2Parser(feature)
+
+    for layout in layouts:
+        spritelayout = (
+            action2.resolve_spritegroup(layout.name) if isinstance(layout, expression.SpriteGroupRef) else None
+        )
+        if not spritelayout or not isinstance(spritelayout, spriteblock.SpriteLayout):
+            raise generic.ScriptError("Expected a SpriteLayout", layout.pos)
+
+        # Allocate registers
+        registers = []
+        param_map = {}
+        for i, param in enumerate(spritelayout.param_list):
+            reg = action2var.VarAction2CallParam(param.value)
+            param_registers.append(reg)
+            param_map[param.value] = reg
+            store_tmp = action2var.VarAction2StoreCallParam(reg)
+            registers.append((reg, store_tmp, layout.param_list[i]))
+        param_map = [(param_map, lambda name, value, pos: action2var.VarAction2LoadCallParam(value, name))]
+        param_map.extend(default_param)
+
+        layout = ParsedSpriteLayout(registers)
+        layout.process(spritelayout, feature, param_map, actions, var10map)
+        layout.parse_registers(varact2parser)
+        parsed_layouts.append(layout)
+
+    actions.extend(action0.get_layout_action0(feature, id, parsed_layouts))
+
+    registers_ref = None
+
+    # Create a procedure only if needed
+    if varact2parser.var_list:
+        # Remove the last VAL2 operator
+        varact2parser.var_list.pop()
+        varact2parser.var_list_size -= 1
+
+        actions.extend(varact2parser.extra_actions)
+        extra_act6 = action6.Action6()
+        for mod in varact2parser.mods:
+            extra_act6.modify_bytes(mod.param, mod.size, mod.offset + 4)
+        if len(extra_act6.modifications) > 0:
+            actions.append(extra_act6)
+
+        varaction2 = action2var.Action2Var(
+            feature, "Station Layout@registers - Id {:02X}".format(id.value), None, 0x89, param_registers
+        )
+        varaction2.var_list = varact2parser.var_list
+        varaction2.default_result = expression.ConstantNumeric(0)
+        varaction2.default_comment = "Return computed value"
+
+        actions.append(varaction2)
+        registers_ref = expression.SpriteGroupRef(expression.Identifier(varaction2.name), [], None, varaction2)
+
+    return (actions, var10map, registers_ref)
